/*
 -------------------------------------------------------------------------------
 | Producer - Copyright (C) 2001-2007  Don Burns                               |
 |                                                                             |
 | This library is free software; you can redistribute it and/or modify        |
 | it under the terms of the GNU Lesser General Public License as published    |
 | by the Free Software Foundation; either version 3 of the License, or        |
 | (at your option) any later version.                                         |
 |                                                                             |
 | This library is distributed in the hope that it will be useful, but         |
 | WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  |
 | or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public     |
 | License for more details.                                                   |
 |                                                                             |
 | You should have received a copy of the GNU Lesser General Public License    |
 | along with this software; if not, write to the Free Software Foundation,    |
 | Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.               |
 |                                                                             |
 -------------------------------------------------------------------------------
 */


#ifndef _PRODUCER_BLOCKING_QUEUE_DEF
#define _PRODUCER_BLOCKING_QUEUE_DEF 1

#include <deque>

#include <Producer/Referenced.h>
#include <Producer/Block.h>

#include <OpenThreads/Mutex>


namespace Producer {

template <class T>
class BlockingQueue : public Referenced, public std::deque<T>
{
    public:
        BlockingQueue() 
        { 
            _block = new Producer::Block;
        }

        void waitWhileEmpty() 
        {
            if( this->empty() )
                _block->reset();
            _block->block();
        }

        void push_back(const T &val)
        {
            OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mut);

#if defined(WIN32) && !defined(__GNUC__)
using namespace std;
            deque<T>::push_back(val);
#else
            std::deque<T>::push_back(val);
#endif
            _block->release();
        }

    protected:
        ~BlockingQueue() {}

    private:
        Producer::ref_ptr<Block> _block;
        OpenThreads::Mutex _mut;
};

}

#endif
