/*
 -------------------------------------------------------------------------------
 | Producer - Copyright (C) 2001-2007  Don Burns                               |
 |                                                                             |
 | This library is free software; you can redistribute it and/or modify        |
 | it under the terms of the GNU Lesser General Public License as published    |
 | by the Free Software Foundation; either version 3 of the License, or        |
 | (at your option) any later version.                                         |
 |                                                                             |
 | This library is distributed in the hope that it will be useful, but         |
 | WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  |
 | or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public     |
 | License for more details.                                                   |
 |                                                                             |
 | You should have received a copy of the GNU Lesser General Public License    |
 | along with this software; if not, write to the Free Software Foundation,    |
 | Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.               |
 |                                                                             |
 -------------------------------------------------------------------------------
 */

#include "QProducer.h"

QProducer::QProducer( QWidget *parent, const char *name )
     : QGLWidget( parent, name ),
            _timer_interval(10),
            _mx(0),
            _my(0),
            _mbutton(0) 
{
}


bool QProducer::realize( Producer::VisualChooser *, Producer::GLContext ) 
{
    startTimer( _timer_interval );
    return _realized = true;
}

bool QProducer::makeCurrent( bool ) 
{
    return true;
}

void QProducer::initializeGL()
{
    _camera = new Producer::Camera;

    _camera->setRenderSurface( this );

    Producer::ref_ptr<Producer::OsgSceneHandler> osgSceneHandler = new Producer::OsgSceneHandler;

    osg::ref_ptr<osg::Node> cow = osgDB::readNodeFile( "cow.osg" );
    if( cow.valid() )
        osgSceneHandler->setScene( cow.get() );

    _camera->setSceneHandler( osgSceneHandler.get() );

    Producer::Vec3 sceneCenter;
    double sceneRadius;
    osgSceneHandler->getSceneBounds( sceneCenter, sceneRadius );

    _trackball = new Producer::Trackball;
    _trackball->setOrientation( Producer::Trackball::Z_UP );
    _trackball->translate(-sceneCenter[0],-sceneCenter[1],-sceneCenter[2]);
    _trackball->setDistance(sceneRadius*3.0f);
    _trackball->setReference();
}


void QProducer::resizeGL( int width, int height )
{
    setWindowRectangle( 0, 0, width, height, false );
}


void QProducer::paintGL()
{
    float nx, ny;
    _normalizeMouseXY( _mx, _my, nx, ny );

    _trackball->input( nx, ny, _mbutton );
    _camera->setViewByMatrix( _trackball->getMatrix() );
    _camera->frame(false);
}

void QProducer::timerEvent(QTimerEvent*)
{
    updateGL();
}

void QProducer::keyPressEvent( QKeyEvent* event )
{
    if( event->ascii() == ' ' )
        _trackball->reset();
}

void QProducer::keyReleaseEvent( QKeyEvent* )
{
}

void QProducer::mousePressEvent( QMouseEvent* event )
{
    switch(event->button())
    {
        case Qt::LeftButton:    _mbutton |= 1; break;
        case Qt::MidButton:     _mbutton |= 2; break;
        case Qt::RightButton:   _mbutton |= 4; break;
        default: break;
    }
    _mx = event->x();
    _my = event->y();
}

void QProducer::mouseReleaseEvent( QMouseEvent* event )
{
    switch(event->button())
    {
        case Qt::LeftButton:    _mbutton &= ~1; break;
        case Qt::MidButton:     _mbutton &= ~2; break;
        case Qt::RightButton:   _mbutton &= ~4; break;
        default: break;
    }

    _mx = event->x();
    _my = event->y();

}

void QProducer::mouseMoveEvent( QMouseEvent* event )
{
    _mx = event->x();
    _my = event->y();
}

void QProducer::_normalizeMouseXY( int x, int y, float &nx, float &ny )
{
    int wx, wy;
    unsigned int ww, wh;
    getWindowRectangle( wx, wy, ww, wh );

    nx = -1.0 + (2 * (float)x/(float)ww);
    ny =  1.0 - (2 * (float)y/(float)wh);
}

